## Méthode Reinert de classification
##
## "Robinson Crusoé - Tome I" de Daniel Defoe (1719),
## trad. fr. Petrus Borel (1836)
## Source : https://fr.feedbooks.com/book/5028/robinson-cruso%C3%A9-tome-i

## Chargement des extensions
library(tidyverse)
library(quanteda)
library(rainette)

## Import des données
txt <- read_file("robinson_crusoe_t1.txt")


# Prétraitement ---------------------------------------

## Création du corpus
corp <- corpus(txt)
corp
## Découpage du texte en segments
corp <- split_segments(corp, segment_size = 40)
corp

## Tokenization
tok <- tokens(corp, remove_punct = TRUE)
tok <- tokens_tolower(tok)
tok <- tokens_split(tok, "'")
mots_outils <- setdiff(
    stopwords("fr"),
    c("je", "j", "t", "tu", "il", "elle", "nous", "vous", "ils", "elles")
)
tok <- tokens_remove(tok, mots_outils)

## Création de la DTM
dtm <- dfm(tok)
dtm <- dfm_trim(dtm, min_docfreq = 5)
dtm


# Classification simple -------------------------------

## Classification en 8 classes
res <- rainette(dtm, k = 8, min_segment_size = 20)
rainette_explor(res, dtm, corp)

rainette_plot(res, dtm, k = 8)

## Récupération des groupes en 8 classes
corp$groupe <- cutree(res, k = 8)
corp$groupe[corp$groupe == 1] <- "mer"
corp$groupe[corp$groupe == 2] <- "construction"
corp$groupe[corp$groupe == 3] <- "agriculture"
corp$groupe[corp$groupe == 4] <- "armes"
corp$groupe[corp$groupe == 5] <- "réflexions"
corp$groupe[corp$groupe == 6] <- "divers"
corp$groupe[corp$groupe == 7] <- "bateau"
corp$groupe[corp$groupe == 8] <- "autre personne"

## Graphique de la position des différentes classes dans le texte
docvars(corp)$position <- seq_len(ndoc(corp))
ggplot(docvars(corp)) +
    geom_density(aes(x = position, fill = groupe), bw = 20) +
    geom_rug(aes(x = position, color = groupe), alpha = 0.5) +
    ggthemes::scale_fill_tableau() +
    ggthemes::scale_color_tableau() +
    facet_grid(rows = vars(groupe), scales = "free_y")


# Classification double -------------------

res1 <- rainette(dtm, k = 8, min_segment_size = 20)
res2 <- rainette(dtm, k = 8, min_segment_size = 40)

res <- rainette2(res1, res2, max_k = 8)

rainette2_explor(res, dtm, corp)

corp$groupe2 <- cutree(res, k = 8)
corp$groupe2[corp$groupe2 == 1] <- "mer"
corp$groupe2[corp$groupe2 == 2] <- "réflexions"
corp$groupe2[corp$groupe2 == 3] <- "armes"
corp$groupe2[corp$groupe2 == 4] <- "construction"
corp$groupe2[corp$groupe2 == 5] <- "agriculture"
corp$groupe2[corp$groupe2 == 6] <- "divers"
corp$groupe2[corp$groupe2 == 7] <- "bateau"
corp$groupe2[corp$groupe2 == 8] <- "autre personne"

ggplot(docvars(corp)) +
    geom_density(aes(x = position, fill = groupe2), bw = 20) +
    geom_rug(aes(x = position, color = groupe2), alpha = 0.5) +
    ggthemes::scale_fill_tableau() +
    ggthemes::scale_color_tableau() +
    facet_grid(rows = vars(groupe2), scales = "free_y")
